@@ -12,7 +12,7 @@
 
 """Here is defined the Index class."""
 
-from __future__ import print_function
+
 import math
 import operator
 import os
@@ -21,30 +21,29 @@ import sys
 import tempfile
 import warnings
 
-from time import time, clock
+from time import time, perf_counter
 
 import numpy
 
-from tables.idxutils import (calc_chunksize, calcoptlevels,
+from .idxutils import (calc_chunksize, calcoptlevels,
                              get_reduction_level, nextafter, inftype)
 
-from tables import indexesextension
-from tables.node import NotLoggedMixin
-from tables.atom import UIntAtom, Atom
-from tables.earray import EArray
-from tables.carray import CArray
-from tables.leaf import Filters
-from tables.indexes import CacheArray, LastRowArray, IndexArray
-from tables.group import Group
-from tables.path import join_path
-from tables.exceptions import PerformanceWarning
-from tables.utils import is_idx, idx2long, lazyattr
-from tables.utilsextension import (nan_aware_gt, nan_aware_ge,
+from . import indexesextension
+from .node import NotLoggedMixin
+from .atom import UIntAtom, Atom
+from .earray import EArray
+from .carray import CArray
+from .leaf import Filters
+from .indexes import CacheArray, LastRowArray, IndexArray
+from .group import Group
+from .path import join_path
+from .exceptions import PerformanceWarning
+from .utils import is_idx, idx2long, lazyattr
+from .utilsextension import (nan_aware_gt, nan_aware_ge,
                                    nan_aware_lt, nan_aware_le,
                                    bisect_left, bisect_right)
-from tables.lrucacheextension import ObjectCache
+from .lrucacheextension import ObjectCache
 
-from tables._past import previous_api, previous_api_property
 
 
 # default version for INDEX objects
@@ -59,7 +58,7 @@ debug = False
 profile = False
 # profile = True  # Uncomment for profiling
 if profile:
-    from tables.utils import show_stats
+    from .utils import show_stats
 
 
 # The default method for sorting
@@ -102,10 +101,8 @@ def _table_column_pathname_of_index(indexpathname):
     colpathname = "/".join(names[i + 1:])
     return (tablepathname, colpathname)
 
-_tableColumnPathnameOfIndex = previous_api(_table_column_pathname_of_index)
 
-
-class Index(NotLoggedMixin, indexesextension.Index, Group):
+class Index(NotLoggedMixin, Group, indexesextension.Index):
     """Represents the index of a column in a table.
 
     This class is used to keep the indexing information for columns in a Table
@@ -159,26 +156,31 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
 
     _c_classid = 'INDEX'
 
-    _c_classId = previous_api_property('_c_classid')
 
-    kind = property(
-        lambda self: {1: 'ultralight', 2: 'light',
-                      4: 'medium', 8: 'full'}[self.indsize],
-        None, None,
-        "The kind of this index.")
+    @property
+    def kind(self):
+        "The kind of this index."
+        return {1: 'ultralight', 2: 'light',
+                4: 'medium', 8: 'full'}[self.indsize]
 
-    filters = property(
-        lambda self: self._v_filters, None, None,
+    @property
+    def filters(self):
         """Filter properties for this index - see Filters in
-        :ref:`FiltersClassDescr`.""")
+        :ref:`FiltersClassDescr`."""
+        return self._v_filters
 
-    def _getdirty(self):
-        if 'DIRTY' not in self._v_attrs:
-            # If there is no ``DIRTY`` attribute, index should be clean.
-            return False
-        return self._v_attrs.DIRTY
+    @property
+    def dirty(self):
+       """Whether the index is dirty or not.
+       Dirty indexes are out of sync with column data, so they exist but they
+       are not usable.
+       """
 
-    def _setdirty(self, dirty):
+       # If there is no ``DIRTY`` attribute, index should be clean.
+       return getattr(self._v_attrs, 'DIRTY', False)
+
+    @dirty.setter
+    def dirty(self, dirty):
         wasdirty, isdirty = self.dirty, bool(dirty)
         self._v_attrs.DIRTY = dirty
         # If an *actual* change in dirtiness happens,
@@ -189,47 +191,42 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
         if wasdirty and not isdirty:
             conditioncache.unnail()
 
-    dirty = property(
-        _getdirty, _setdirty, None,
-        """Whether the index is dirty or not.
+    @property
+    def column(self):
+        """The Column (see :ref:`ColumnClassDescr`) instance for the indexed
+        column."""
 
-        Dirty indexes are out of sync with column data, so they exist but they
-        are not usable.
-        """)
-
-    def _getcolumn(self):
         tablepath, columnpath = _table_column_pathname_of_index(
             self._v_pathname)
         table = self._v_file._get_node(tablepath)
         column = table.cols._g_col(columnpath)
         return column
 
-    column = property(_getcolumn, None, None,
-        """The Column (see :ref:`ColumnClassDescr`) instance for the indexed
-        column.""")
-
-    def _gettable(self):
-        tablepath, columnpath = _table_column_pathname_of_index(
-            self._v_pathname)
+    @property
+    def table(self):
+        """Accessor for the `Table` object of this index."""
+        tablepath, columnpath = _table_column_pathname_of_index(self._v_pathname)
         table = self._v_file._get_node(tablepath)
         return table
 
-    table = property(_gettable, None, None,
-                     "Accessor for the `Table` object of this index.")
+    @property
+    def nblockssuperblock(self):
+        "The number of blocks in a superblock."
+        return self.superblocksize // self.blocksize
 
-    nblockssuperblock = property(
-        lambda self: self.superblocksize // self.blocksize, None, None,
-        "The number of blocks in a superblock.")
+    @property
+    def nslicesblock(self):
+        "The number of slices in a block."
+        return self.blocksize // self.slicesize
 
-    nslicesblock = property(
-        lambda self: self.blocksize // self.slicesize, None, None,
-        "The number of slices in a block.")
+    @property
+    def nchunkslice(self):
+        "The number of chunks in a slice."
+        return self.slicesize // self.chunksize
 
-    nchunkslice = property(
-        lambda self: self.slicesize // self.chunksize, None, None,
-        "The number of chunks in a slice.")
-
-    def _g_nsuperblocks(self):
+    @property
+    def nsuperblocks(self):
+        "The total number of superblocks in index."
         # Last row should not be considered as a superblock
         nelements = self.nelements - self.nelementsILR
         nblocks = nelements // self.superblocksize
@@ -237,10 +234,9 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
             nblocks += 1
         return nblocks
 
-    nsuperblocks = property(_g_nsuperblocks, None, None,
-                            "The total number of superblocks in index.")
-
-    def _g_nblocks(self):
+    @property
+    def nblocks(self):
+        "The total number of blocks in index."
         # Last row should not be considered as a block
         nelements = self.nelements - self.nelementsILR
         nblocks = nelements // self.blocksize
@@ -248,34 +244,40 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
             nblocks += 1
         return nblocks
 
-    nblocks = property(_g_nblocks, None, None,
-                       "The total number of blocks in index.")
+    @property
+    def nslices(self):
+        "The number of complete slices in index."
+        return self.nelements // self.slicesize
 
-    nslices = property(
-        lambda self: self.nelements // self.slicesize, None, None,
-        "The number of complete slices in index.")
+    @property
+    def nchunks(self):
+        "The number of complete chunks in index."
+        return self.nelements // self.chunksize
 
-    nchunks = property(
-        lambda self: self.nelements // self.chunksize, None, None,
-        "The number of complete chunks in index.")
+    @property
+    def shape(self):
+        "The shape of this index (in slices and elements)."
+        return (self.nrows, self.slicesize)
 
-    shape = property(
-        lambda self: (self.nrows, self.slicesize), None, None,
-        "The shape of this index (in slices and elements).")
+    @property
+    def temp_required(self):
+        "Whether a temporary file for indexes is required or not."
+        return self.indsize > 1 and self.optlevel > 0 and self.table.nrows > self.slicesize
 
-    temp_required = property(
-        lambda self: (self.indsize > 1 and
-                      self.optlevel > 0 and
-                      self.table.nrows > self.slicesize),
-        None, None,
-        "Whether a temporary file for indexes is required or not.")
+    @property
+    def want_complete_sort(self):
+        "Whether we should try to build a completely sorted index or not."
+        return self.indsize == 8 and self.optlevel == 9
 
-    want_complete_sort = property(
-        lambda self: (self.indsize == 8 and self.optlevel == 9),
-        None, None,
-        "Whether we should try to build a completely sorted index or not.")
+    @property
+    def is_csi(self):
+        """Whether the index is completely sorted or not.
+
+        .. versionchanged:: 3.0
+           The *is_CSI* property has been renamed into *is_csi*.
+
+        """
 
-    def _is_csi(self):
         if self.nelements == 0:
             # An index with 0 indexed elements is not a CSI one (by definition)
             return False
@@ -290,18 +292,6 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
         self.compute_overlaps(self, None, False)
         return self.noverlaps == 0
 
-    _is_CSI = previous_api(_is_csi)
-
-    is_csi = property(_is_csi, None, None,
-        """Whether the index is completely sorted or not.
-
-        .. versionchanged:: 3.0
-           The *is_CSI* property has been renamed into *is_csi*.
-
-        """)
-
-    is_CSI = previous_api(is_csi)
-
     @lazyattr
     def nrowsinchunk(self):
         """The number of rows that fits in a *table* chunk."""
@@ -398,7 +388,7 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
         self.tprof = 0
         """Time counter for benchmarking purposes."""
 
-        from tables.file import open_file
+        from .file import open_file
         self._openFile = open_file
         """The `open_file()` function, to avoid a circular import."""
 
@@ -558,7 +548,6 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
         if self.temp_required:
             self.create_temp()
 
-    _g_postInitHook = previous_api(_g_post_init_hook)
 
     def initial_append(self, xarr, nrow, reduction):
         """Compute an initial indices arrays for data to be indexed."""
@@ -599,7 +588,7 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
             idx = numpy.empty(len(arr), "uint%d" % (indsize * 8))
             lbucket = self.lbucket
             # Fill the idx with the bucket indices
-            offset = lbucket - ((nrow * (slicesize % lbucket)) % lbucket)
+            offset = int(lbucket - ((nrow * (slicesize % lbucket)) % lbucket))
             idx[0:offset] = 0
             for i in range(offset, slicesize, lbucket):
                 idx[i:i + lbucket] = (i + lbucket - 1) // lbucket
@@ -772,7 +761,6 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
         if profile:
             show_stats("Exiting appendLR", tref)
 
-    appendLastRow = previous_api(append_last_row)
 
     def optimize(self, verbose=False):
         """Optimize an index so as to allow faster searches.
@@ -854,7 +842,7 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
 
         if self.verbose:
             t1 = time()
-            c1 = clock()
+            c1 = perf_counter()
         ss = self.slicesize
         tmp = self.tmp
         ranges = tmp.ranges[:]
@@ -960,7 +948,7 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
         self.compute_overlaps(self.tmp, "do_complete_sort()", self.verbose)
         if self.verbose:
             t = round(time() - t1, 4)
-            c = round(clock() - c1, 4)
+            c = round(perf_counter() - c1, 4)
             print("time: %s. clock: %s" % (t, c))
 
     def swap(self, what, mode=None):
@@ -975,7 +963,7 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
 
         if self.verbose:
             t1 = time()
-            c1 = clock()
+            c1 = perf_counter()
         if what == "chunks":
             self.swap_chunks(mode)
         elif what == "slices":
@@ -989,7 +977,7 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
         rmult = len(mult.nonzero()[0]) / float(len(mult))
         if self.verbose:
             t = round(time() - t1, 4)
-            c = round(clock() - c1, 4)
+            c = round(perf_counter() - c1, 4)
             print("time: %s. clock: %s" % (t, c))
         # Check that entropy is actually decreasing
         if what == "chunks" and self.last_tover > 0. and self.last_nover > 0:
@@ -1233,7 +1221,7 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
                 ul = self.nelementsILR // cs
                 bounds = numpy.concatenate((bounds, self.bebounds[:ul]))
             sbounds_idx = bounds.argsort(kind=defsort)
-            offset = nblock * nsb
+            offset = int(nblock * nsb)
             # Swap sorted and indices following the new order
             self.get_neworder(sbounds_idx, sorted, tmp_sorted, sortedLR,
                               nslices, offset, self.dtype)
@@ -1280,7 +1268,6 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
         stepl = numpy.array([1], dtype=numpy.uint64)
         where._g_write_slice(startl, stepl, countl, buffer)
 
-    read_sliceLR = previous_api(read_slice_lr)
 
     def reorder_slice(self, nslice, sorted, indices, ssorted, sindices,
                       tmp_sorted, tmp_indices):
@@ -1740,7 +1727,6 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
 
         return self.read_sorted_indices('sorted', start, stop, step)
 
-    readSorted = previous_api(read_sorted)
 
     def read_indices(self, start=None, stop=None, step=None):
         """Return the indices values of index in the specified range.
@@ -1752,7 +1738,6 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
 
         return self.read_sorted_indices('indices', start, stop, step)
 
-    readIndices = previous_api(read_indices)
 
     def _process_range(self, start, stop, step):
         """Get a range specifc for the index usage."""
@@ -1774,7 +1759,6 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
             step = idx2long(step)
         return (start, stop, step)
 
-    _processRange = previous_api(_process_range)
 
     def __getitem__(self, key):
         """Return the indices values of index in the specified range.
@@ -2016,7 +2000,6 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
             stop = 0
         return (start, stop)
 
-    searchLastRow = previous_api(search_last_row)
 
     def get_chunkmap(self):
         """Compute a map with the interesting chunks in index."""
@@ -2126,7 +2109,6 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
 
         return range_
 
-    getLookupRange = previous_api(get_lookup_range)
 
     def _f_remove(self, recursive=False):
         """Remove this Index object."""
@@ -2143,6 +2125,8 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
         if self.filters.complevel:
             if self.filters.shuffle:
                 filters += ", shuffle"
+            if self.filters.bitshuffle:
+                filters += ", bitshuffle"
             filters += ", %s(%s)" % (self.filters.complib,
                                      self.filters.complevel)
         return "Index(%s, %s%s).is_csi=%s" % \
@@ -2182,7 +2166,6 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
 class IndexesDescG(NotLoggedMixin, Group):
     _c_classid = 'DINDEX'
 
-    _c_classId = previous_api_property('_c_classid')
 
     def _g_width_warning(self):
         warnings.warn(
@@ -2192,25 +2175,25 @@ class IndexesDescG(NotLoggedMixin, Group):
             "and possibly slow I/O" % self._v_max_group_width,
             PerformanceWarning)
 
-    _g_widthWarning = previous_api(_g_width_warning)
 
 
 class IndexesTableG(NotLoggedMixin, Group):
     _c_classid = 'TINDEX'
 
-    _c_classId = previous_api_property('_c_classid')
 
-    def _getauto(self):
+    @property
+    def auto(self):
         if 'AUTO_INDEX' not in self._v_attrs:
             return default_auto_index
         return self._v_attrs.AUTO_INDEX
 
-    def _setauto(self, auto):
+    @auto.setter
+    def auto(self, auto):
         self._v_attrs.AUTO_INDEX = bool(auto)
 
-    def _delauto(self):
+    @auto.deleter
+    def auto(self):
         del self._v_attrs.AUTO_INDEX
-    auto = property(_getauto, _setauto, _delauto)
 
     def _g_width_warning(self):
         warnings.warn(
@@ -2220,16 +2203,16 @@ class IndexesTableG(NotLoggedMixin, Group):
             "and possibly slow I/O" % self._v_max_group_width,
             PerformanceWarning)
 
-    _g_widthWarning = previous_api(_g_width_warning)
 
     def _g_check_name(self, name):
         if not name.startswith('_i_'):
             raise ValueError(
                 "names of index groups must start with ``_i_``: %s" % name)
 
-    _g_checkName = previous_api(_g_check_name)
 
-    def _gettable(self):
+    @property
+    def table(self):
+        "Accessor for the `Table` object of this `IndexesTableG` container."
         names = self._v_pathname.split("/")
         tablename = names.pop()[3:]   # "_i_" is at the beginning
         parentpathname = "/".join(names)
@@ -2237,17 +2220,12 @@ class IndexesTableG(NotLoggedMixin, Group):
         table = self._v_file._get_node(tablepathname)
         return table
 
-    table = property(
-        _gettable, None, None,
-        "Accessor for the `Table` object of this `IndexesTableG` container.")
-
 
 class OldIndex(NotLoggedMixin, Group):
     """This is meant to hide indexes of PyTables 1.x files."""
 
     _c_classid = 'CINDEX'
 
-    _c_classId = previous_api_property('_c_classid')
 
 
 ## Local Variables:
