@@ -1,7 +1,5 @@
 # -*- coding: utf-8 -*-
 
-from __future__ import print_function
-from __future__ import absolute_import
 
 import os
 import sys
@@ -19,7 +17,6 @@ from tables.tests import common
 from tables.tests.common import allequal
 from tables.tests.common import unittest, test_filename
 from tables.tests.common import PyTablesTestCase as TestCase
-from six.moves import range
 
 
 #warnings.resetwarnings()
@@ -302,7 +299,7 @@ class BasicTestCase(TestCase):
                 else:
                     # If a is not a python string, then it should be a list
                     # or ndarray
-                    self.assertTrue(type(b) in [list, numpy.ndarray])
+                    self.assertIn(type(b), [list, numpy.ndarray])
         finally:
             # Then, delete the file
             os.remove(filename)
@@ -327,7 +324,7 @@ class BasicTestCase(TestCase):
                                       lambda: fileh.root.somearray.read(out=b))
                 else:
                     fileh.root.somearray.read(out=b)
-                self.assertTrue(type(b), numpy.ndarray)
+                self.assertIsInstance(b, numpy.ndarray)
         finally:
             # Then, delete the file
             os.remove(filename)
@@ -366,7 +363,7 @@ class BasicTestCase(TestCase):
                                       lambda: fileh.root.somearray.read(out=b))
                 else:
                     fileh.root.somearray.read(out=b)
-                self.assertTrue(type(b), numpy.ndarray)
+                self.assertIsInstance(b, numpy.ndarray)
         finally:
             # Then, delete the file
             os.remove(filename)
@@ -610,7 +607,7 @@ class ReadOutArgumentTests(common.TempFileMixin, TestCase):
         try:
             disk_array.read(0, self.size, 1, out_buffer)
         except ValueError as exc:
-            self.assertTrue('output array size invalid, got' in str(exc))
+            self.assertIn('output array size invalid, got', str(exc))
 
     def test_buffer_too_large(self):
         array, disk_array = self.create_array()
@@ -620,7 +617,7 @@ class ReadOutArgumentTests(common.TempFileMixin, TestCase):
         try:
             disk_array.read(0, self.size, 1, out_buffer)
         except ValueError as exc:
-            self.assertTrue('output array size invalid, got' in str(exc))
+            self.assertIn('output array size invalid, got', str(exc))
 
 
 class SizeOnDiskInMemoryPropertyTestCase(common.TempFileMixin, TestCase):
@@ -2252,10 +2249,9 @@ class FancySelectionTestCase(common.TempFileMixin, TestCase):
 
         # Using booleans instead of ints is deprecated since numpy 1.8
         # Tests for keys that have to support the __index__ attribute
-        #if (sys.version_info[0] >= 2 and sys.version_info[1] >= 5):
-        #    self.working_keyset.append(
-        #        (False, True),  # equivalent to (0,1) ;-)
-        #    )
+        #self.working_keyset.append(
+        #    (False, True),  # equivalent to (0,1) ;-)
+        #)
 
         # Valid selections for NumPy, but not for PyTables (yet)
         # The next should raise an IndexError
